@@ -3,13 +3,13 @@ import trio
 import anyio
 import time
 
-from .mock_mqtt import stdtest
-
-from .run import run
+from distkv.mock.mqtt import stdtest
 
 from distkv.code import CodeRoot
 from distkv.runner import SingleRunnerRoot
 from distkv.errors import ErrorRoot
+from distkv.util import P
+
 import logging
 
 logger = logging.getLogger(__name__)
@@ -18,19 +18,13 @@ logger = logging.getLogger(__name__)
 @pytest.mark.trio
 async def test_83_run(autojump_clock):  # pylint: disable=unused-argument
     async with stdtest(args={"init": 123}, tocks=100) as st:
-        s, = st.s
-        h = p = None  # pylint
-        for h, p, *_ in s.ports:
-            if h[0] != ":":
-                break
         async with st.client() as c:
             await ErrorRoot.as_handler(c)
             cr = await CodeRoot.as_handler(c)
             r = await SingleRunnerRoot.as_handler(c, subpath=(), code=cr)
             c._test_evt = anyio.create_event()
             await cr.add(
-                "forty",
-                "two",
+                P("forty.two"),
                 code="""\
                 import trio
                 c=_client
@@ -40,29 +34,18 @@ async def test_83_run(autojump_clock):  # pylint: disable=unused-argument
                 """,
                 is_async=True,
             )
-            ru = r.follow("testnode", "foo", "test", create=True)
-            ru.code = ("forty", "two")
+            ru = r.follow(P("testnode.foo.test"), create=True)
+            ru.code = P("forty.two")
             await ru.run_at(time.time())
             logger.info("Start sleep")
             with trio.fail_after(60):
                 await c._test_evt.wait()
-            await run(
-                "-vvv",
-                "client",
-                "-h",
-                h,
-                "-p",
-                p,
-                "data",
-                "get",
-                "-rd_",
-                do_stdout=False,
-            )
+            await st.run("data get -rd_ :", do_stdout=False)
             await trio.sleep(11)
 
             logger.info("End sleep")
 
-            r = await run("-vvv", "client", "-h", h, "-p", p, "data", "get")
+            r = await st.run("data get :")
             assert r.stdout == "123\n"
 
             rs = ru.state
